* 8/31/21
* NOTEs: -Stata 15 required for the "threshold" command. 
*	     -May also need to download user-written 'kpss' command
*		 -will need to load in Hansen .ado files
* ----------------------------------------------------------------------

clear mata

set seed 5829589

* ---------------------------------------------------------------------
* ---------------------------------------------------------------------
* Example I: Soroka, Stecula, and Wlezien (2015)
* open up their data
import delimited "EconomicNewsAJPS.txt", delimiter(tab) clear
sort year month 
gen counter = _n
tsset counter

gen ltone1 = l.tone1
gen lhp_cblagus = l.hp_cblagus
gen lhp_cbcoinus = l.hp_cbcoinus
gen lhp_cb_leadrevised = l.hp_cb_leadrevised

*test for stationarity:
dfuller tone1, lags(10)  // I(0)
dfuller hp_cblagus, lags(10)  // I(0)
dfuller hp_cbcoinus, lags(10) // I(0)
dfuller hp_cb_leadrevised, lags(10) // I(0)

dfgls tone1, maxlag(10) // I(0) (9 lags)
dfgls hp_cblagus, maxlag(10) // I(0) (9 lags)
dfgls hp_cbcoinus, maxlag(10) // I(0) 9 lags)
dfgls hp_cb_leadrevised, maxlag(10) // I(0) (9 lags)

kpss tone1 // I(0) (9 lags)
kpss hp_cblagus // I(0) (9 lags)
kpss hp_cbcoinus // I(0) (9 lags)
kpss hp_cb_leadrevised // I(0) (9 lags)

* Test for non-linearity:
reg tone1 l.tone1 hp_cblagus l.hp_cblagus hp_cbcoinus l.hp_cbcoinus hp_cb_leadrevised l.hp_cb_leadrevised
predict resid, res
predict yhat,

* McLeod and Li (1)
corrgram resid // rejecting H0 of the Ljung-Box Q stat suggests non-linear effects

* McLeod and Li (2) LM-style test
gen e2 = resid^2
tsline e2
reg e2 l.e2
test l.e2 // rejecting the null suggests non-linearity
reg e2 l.e2 l2.e2
test l.e2 l2.e2
reg e2 l.e2 l2.e2 l3.e2
test l.e2 l2.e2 l3.e2
* note that this should be similar to:
qui reg tone1 l.tone1 hp_cblagus l.hp_cblagus hp_cbcoinus l.hp_cbcoinus hp_cb_leadrevised l.hp_cb_leadrevised
estat archlm, lags(1/3)

* RESET test:
forv i = 2/5 {
	gen yhat`i' = yhat^`i'
}
* For H=5
reg resid l.tone1 hp_cblagus l.hp_cblagus hp_cbcoinus l.hp_cbcoinus hp_cb_leadrevised l.hp_cb_leadrevised yhat2 yhat3 yhat4 yhat5
test yhat2 yhat3 yhat4 yhat5 // rejecting H0 suggests non-linearity
* For H=4
reg resid l.tone1 hp_cblagus l.hp_cblagus hp_cbcoinus l.hp_cbcoinus hp_cb_leadrevised l.hp_cb_leadrevised yhat2 yhat3 yhat4
test yhat2 yhat3 yhat4 // rejecting H0 suggests non-linearity
* For H = 3
reg resid l.tone1 hp_cblagus l.hp_cblagus hp_cbcoinus l.hp_cbcoinus hp_cb_leadrevised l.hp_cb_leadrevised yhat2 yhat3 
test yhat2 yhat3 

* Stationarity testing:
* step 1: regress series on a constant, save resids:
reg tone1 
predict tone1_star, resid
* step 2: estimate threshold model of d.y on lag of y with l.y determining threshold and no constant:
threshold d.tone1_star, nocons threshvar(l.tone1_star) regionvars(l.tone1_star)
* Since we need to use the F test (not chi2), just going to create the model above by hand and use t statistics so we can use the f test
gen ltone1_star_lo = l.tone1_star
gen ltone1_star_hi = l.tone1_star
replace ltone1_star_lo = 0 if l.tone1_star >  -0.4403
replace ltone1_star_hi = 0 if l.tone1_star <=  -0.4403
reg d.tone1_star ltone1_star_lo ltone1_star_hi, nocon
* step 3: test that rho1 = rho2 = 0
test ltone1_star_lo ltone1_star_hi // critical values for T = 250: 4.56 (95%), 6.47 (99%). 
* so we reject null of unit root in favor of TAR process

* Given that everything's stationary, we proceed to run an ARDL model. TABLE 1, MODEL 1:
reg tone1 l.tone1 hp_cblagus l.hp_cblagus hp_cbcoinus l.hp_cbcoinus hp_cb_leadrevised l.hp_cb_leadrevised
estat ic
est sto ardl
/* calculate AIC/BIC by hand using SSR instead of likelihood. Stata gives this as:
	local aic = `T'*log(`finalssr'/`T') + 2*`nparams'
	local bic = `T'*log(`finalssr'/`T') + log(`T')*`nparams'
*/
di 383*log(28.1099147/383) + 2*8 // AIC -984.36254
di 383*log(28.1099147/383) + log(383)*8 // BIC -952.77826


* Estimate threshold model, threshold determined by l.tone1. TABLE 1, MODEL 2
threshold tone1, threshvar(l.tone1) regionvars(l.tone1 hp_cblagus l.hp_cblagus hp_cbcoinus l.hp_cbcoinus hp_cb_leadrevised l.hp_cb_leadrevised) trim(10) ssrs(tar)
est sto tarmodel 



* here's the plot of the SSR against the threshold var. FIGURE 2
sort ltone1
twoway line tar ltone1 if ltone1 >= -.2 & ltone1 <= .7, ytitle("SSR") yline(26.3389) xtitle("Lagged Media Tone") xlabel(-.2(.1).6) lwidth(medthick)
*graph export "soroka_SSR.pdf", as(pdf) replace


* Now load in Hansen's programs 
do "thresholdreg.ado"
do "thresholdtest.ado"

* need to run this all at once starting w/ preserve and ending w/ restore. Hansen's programs take some time to run
preserve
tsset
threshold tone1, threshvar(l.tone1) regionvars(l.tone1 hp_cblagus l.hp_cblagus hp_cbcoinus l.hp_cbcoinus hp_cb_leadrevised l.hp_cb_leadrevised) trim(10)
keep if e(sample)
tsset

* Now use Hansen's programs:
* Hansen's for threshold model vs. linear model:
thresholdtest tone1 ltone1 hp_cblagus lhp_cblagus hp_cbcoinus lhp_cbcoinus hp_cb_leadrevised lhp_cb_leadrevised, q(ltone1) trim_per(.10) rep(1000)

* Now try the threshold regression:
thresholdreg tone1 ltone1 hp_cblagus lhp_cblagus hp_cbcoinus lhp_cbcoinus hp_cb_leadrevised lhp_cb_leadrevised, q(ltone1) h(1) // impose heteroskedasticity

thresholdreg tone1 ltone1 hp_cblagus lhp_cblagus hp_cbcoinus lhp_cbcoinus hp_cb_leadrevised lhp_cb_leadrevised, q(ltone1) h(0) // impose homoskedasticity
* these results should be very similar Stata's
* Create our own plot:
gen threshpoint_y = .
replace threshpoint_y = -1 in 1
gen threshpoint_x = .
replace threshpoint_x = -.0261 in 1
gen threshpoint_lo = .
replace threshpoint_lo = -.269843191 in 1
gen threshpoint_hi = .
replace threshpoint_hi = .205928579 in 1

* FIGURE 3
twoway line lr_est qs_est, yline(7.3523) xtitle("Lagged Media Tone") ytitle("Likelihood Ratio Value") || scatter threshpoint_y threshpoint_x, mcolor(black) || rcap threshpoint_hi threshpoint_lo threshpoint_y, horizontal legend(off) lcolor(black) lwidth(medthick)
*graph export "soroka_95CIs.pdf", as(pdf) replace
restore

* Robustness: Test to see whether our choice of threshold given by l.tone1 is valid up to 3 lags of tone:
tsset
threshold tone1, threshvar(l3.tone1) regionvars(l.tone1 hp_cblagus l.hp_cblagus hp_cbcoinus l.hp_cbcoinus hp_cb_leadrevised l.hp_cb_leadrevised) trim(10)
gen lagtest = e(sample)
*AIC              =  -983.9823
*BIC              =  -920.8976
threshold tone1 if lagtest == 1, threshvar(l2.tone1) regionvars(l.tone1 hp_cblagus l.hp_cblagus hp_cbcoinus l.hp_cbcoinus hp_cb_leadrevised l.hp_cb_leadrevised) trim(10)
*AIC              =   -985.7060
*BIC              =   -922.6212
threshold tone1 if lagtest == 1, threshvar(l1.tone1) regionvars(l.tone1 hp_cblagus l.hp_cblagus hp_cbcoinus l.hp_cbcoinus hp_cb_leadrevised l.hp_cb_leadrevised) trim(10)
*AIC              =   -988.2000
*BIC              =   -925.1152
* so lag of 1 seems to be best

* Band TAR: ----------------------------------------------------
* Impose parameter equality across the high and low regions: 
constraint 1 [Region1]ltone1 = [Region3]ltone1
constraint 2 [Region1]hp_cblagus = [Region3]hp_cblagus
constraint 3 [Region1]lhp_cblagus = [Region3]lhp_cblagus
constraint 4 [Region1]hp_cbcoinus = [Region3]hp_cbcoinus
constraint 5 [Region1]lhp_cbcoinus = [Region3]lhp_cbcoinus
constraint 6 [Region1]hp_cb_leadrevised = [Region3]hp_cb_leadrevised
constraint 7 [Region1]lhp_cb_leadrevised = [Region3]lhp_cb_leadrevised
 
* TABLE 1, MODEL 3
threshold tone1, threshvar(l.tone1) regionvars(ltone1 hp_cblagus lhp_cblagus hp_cbcoinus lhp_cbcoinus hp_cb_leadrevised lhp_cb_leadrevised) trim(10) vce(robust) constraint(1 2 3 4 5 6 7) nthresholds(2)



gen edate = ym(year, month)
format edate %tm
tsset edate
* FIGURE 4
tsline tone1, lwidth(medthick) yline(.5189 -.0261) title("") ytitle("Media Tone") xtitle("Time") 
*graph export "soroka_btar.pdf", as(pdf) replace




* -----------------------------
* EFFECT SIZES
* -----------------------------
* create effect sizes; * Plot the results for leading EI
gen sr = . // create contemporaneous coef placeholder
gen sr_ul = . // create contemporaneous Upper 95% CI placeholder
gen sr_ll = . // create contemporaneous Lower 95% CI placeholder
gen lr = . // create long-run coef placeholder
gen lr_ul = . // create long-run Upper 95% CI placeholder
gen lr_ll = . // create long-run Lower 95% CI placeholder
gen order = _n  // 1 = ARDL, 2 = TAR (lower), 3 = TAR (upper), 4 = B-TAR (in band), 5 = B-TAR (out of band)
replace order = .5 in 6
replace order = 5.5 in 7
replace order = . in 8/383

reg tone1 ltone1 hp_cblagus lhp_cblagus hp_cbcoinus lhp_cbcoinus hp_cb_leadrevised lhp_cb_leadrevised

* obtain SR effects and CIs
replace sr =  _b[hp_cb_leadrevised] if order == 1
replace  sr_ul = _b[hp_cb_leadrevised] + 1.96*_se[hp_cb_leadrevised] if order == 1
replace  sr_ll = _b[hp_cb_leadrevised] - 1.96*_se[hp_cb_leadrevised] if order == 1

* obtain LR effects and CIs
nlcom (_b[hp_cb_leadrevised] + _b[lhp_cb_leadrevised])/(1-_b[ltone1])
mat b = r(b)
mat list b
scalar beta = b[1,1]
replace lr = beta if order == 1
mat SE = r(V)
scalar se = sqrt(SE[1,1])
replace lr_ul = beta + 1.96*se if order == 1
replace lr_ll = beta - 1.96*se if order == 1

threshold tone1, threshvar(ltone1) regionvars(ltone1 hp_cblagus lhp_cblagus hp_cbcoinus lhp_cbcoinus hp_cb_leadrevised lhp_cb_leadrevised)

* obtain SR effects and CIs (lower threshold)
replace sr =  [Region1]_b[hp_cb_leadrevised] if order == 2
replace  sr_ul = [Region1]_b[hp_cb_leadrevised] + 1.96*[Region1]_se[hp_cb_leadrevised] if order == 2
replace  sr_ll = [Region1]_b[hp_cb_leadrevised] - 1.96*[Region1]_se[hp_cb_leadrevised] if order == 2

* obtain LR effects and CIs (lower threshold)
nlcom ([Region1]_b[hp_cb_leadrevised] + [Region1]_b[lhp_cb_leadrevised])/(1-[Region1]_b[ltone1])
mat b = r(b)
mat list b
scalar beta = b[1,1]
replace lr = beta if order == 2
mat SE = r(V)
scalar se = sqrt(SE[1,1])
replace lr_ul = beta + 1.96*se if order == 2
replace lr_ll = beta - 1.96*se if order == 2

* obtain SR effects and CIs (upper threshold)
replace sr =  [Region2]_b[hp_cb_leadrevised] if order == 3
replace  sr_ul = [Region2]_b[hp_cb_leadrevised] + 1.96*[Region2]_se[hp_cb_leadrevised] if order == 3
replace  sr_ll = [Region2]_b[hp_cb_leadrevised] - 1.96*[Region2]_se[hp_cb_leadrevised] if order == 3

* obtain LR effects and CIs (upper threshold)
nlcom ([Region2]_b[hp_cb_leadrevised] + [Region2]_b[lhp_cb_leadrevised])/(1-[Region2]_b[ltone1])
mat b = r(b)
mat list b
scalar beta = b[1,1]
replace lr = beta if order == 3
mat SE = r(V)
scalar se = sqrt(SE[1,1])
replace lr_ul = beta + 1.96*se if order == 3
replace lr_ll = beta - 1.96*se if order == 3


threshold tone1, threshvar(ltone1) regionvars(ltone1 hp_cblagus lhp_cblagus hp_cbcoinus lhp_cbcoinus hp_cb_leadrevised lhp_cb_leadrevised) trim(10) vce(robust) constraint(1 2 3 4 5 6 7) nthresholds(2)

* obtain SR effects and CIs (In band)
replace sr =  [Region2]_b[hp_cb_leadrevised] if order == 4
replace  sr_ul = [Region2]_b[hp_cb_leadrevised] + 1.96*[Region2]_se[hp_cb_leadrevised] if order == 4
replace  sr_ll = [Region2]_b[hp_cb_leadrevised] - 1.96*[Region2]_se[hp_cb_leadrevised] if order == 4

* obtain LR effects and CIs (In band)
nlcom ([Region2]_b[hp_cb_leadrevised] + [Region2]_b[lhp_cb_leadrevised])/(1-[Region2]_b[ltone1])
mat b = r(b)
mat list b
scalar beta = b[1,1]
replace lr = beta if order == 4
mat SE = r(V)
scalar se = sqrt(SE[1,1])
replace lr_ul = beta + 1.96*se if order == 4
replace lr_ll = beta - 1.96*se if order == 4

* obtain SR effects and CIs (out of band)
replace sr =  [Region1]_b[hp_cb_leadrevised] if order == 5
replace  sr_ul = [Region1]_b[hp_cb_leadrevised] + 1.96*[Region1]_se[hp_cb_leadrevised] if order == 5
replace  sr_ll = [Region1]_b[hp_cb_leadrevised] - 1.96*[Region1]_se[hp_cb_leadrevised] if order == 5

* obtain LR effects and CIs (out of band)
nlcom ([Region1]_b[hp_cb_leadrevised] + [Region1]_b[lhp_cb_leadrevised])/(1-[Region1]_b[ltone1])
mat b = r(b)
mat list b
scalar beta = b[1,1]
replace lr = beta if order == 5
mat SE = r(V)
scalar se = sqrt(SE[1,1])
replace lr_ul = beta + 1.96*se if order == 5
replace lr_ll = beta - 1.96*se if order == 5

* get same model estimates closer:
replace order = 2.5 if order == 3
replace order = 3.5 if order == 4
replace order = 4 if order == 5
replace order = 4.5 if order == 5.5

* graph everything. Need to run as a single code chunk from preserve to restore
preserve
drop if lr_ll == .
tsset, clear
twoway rspike sr_ul sr_ll order, lwidth(medthick)  || scatter  sr order, xlabel(1 "ARDL" 2 "TAR (lo)" 2.5 "TAR (hi)" 3.5 "B-TAR (in band)" 4 "B-TAR (out of band)") title("Short-Run Effect") legend(off) xtitle("") yline(0, lcolor(black) lpattern(solid)) msize(medlarge) xlabel(, angle(45)) 
graph save g1.gph, replace
twoway rspike lr_ul lr_ll order, lwidth(medthick) || scatter lr order, title("Long-Run Effect") xlabel(1 "ARDL" 2 "TAR (lo)" 2.5 "TAR (hi)" 3.5 "B-TAR (in band)" 4 "B-TAR (out of band)") legend(off) xtitle("") yline(0, lcolor(black) lpattern(solid)) xlabel(, angle(45)) msize(medlarge) msymbol(S)
graph save g2.gph, replace

* FIGURE 5
graph combine g1.gph g2.gph, ycommon
*graph export "SSW_predictions.pdf", as(pdf) replace
restore








